import uuid
from django.test import TestCase
from django.test.client import RequestFactory
from django.contrib.auth.models import User
from django.contrib.sessions.middleware import SessionMiddleware
from lang_pref_middleware.middleware import LanguagePreferenceMiddleware


class LangPrefMiddlewareTestCaseMixin():
    middleware_class = None

    def setUp(self):
        # pylint: disable=not-callable
        self.middleware = self.middleware_class()
        self.session_middleware = SessionMiddleware()
        self.user = self.get_user()
        self.request = RequestFactory().get('/somewhere')
        self.request.user = self.user
        self.session_middleware.process_request(self.request)

    def get_user(self):
        raise NotImplementedError

    def set_user_language_preference(self, user, language):
        raise NotImplementedError

    def test_no_language_set_in_session_or_prefs(self):
        """
        If no language set in session or user preferences, no language
        should be set in the session.
        """
        self.middleware.process_request(self.request)
        self.assertNotIn('django_language', self.request.session)

    def test_language_in_user_prefs(self):
        """
        If user has a preferred language set, and no language is already
        set on the session, the user's preferred language should be set
        on the session.
        """
        self.set_user_language_preference(self.user, 'eo')
        self.middleware.process_request(self.request)
        self.assertEqual(self.request.session['django_language'], 'eo')

    def test_language_in_session(self):
        """
        If the session already has a language set, it should not be changed
        by the middleware.
        """
        self.request.session['django_language'] = 'en'
        self.set_user_language_preference(self.user, 'eo')
        self.middleware.process_request(self.request)

        self.assertEqual(self.request.session['django_language'], 'en')


class DummyLanguagePreferenceMiddleware(LanguagePreferenceMiddleware):
    """
    Simplified implementation of LanguagePreferenceMiddleware.

    This should not be used for any purpose outside of testing.
    """

    def __init__(self):
        self._cache = {}
        super().__init__()

    def get_user_language_preference(self, user):
        return self._cache.get(user, None)


class LangPrefMiddlewareTests(LangPrefMiddlewareTestCaseMixin, TestCase):
    middleware_class = DummyLanguagePreferenceMiddleware

    def get_user(self):
        username = uuid.uuid4().hex[0:20]
        return User.objects.create_user(username)

    def set_user_language_preference(self, user, language):
        # pylint: disable=protected-access
        self.middleware._cache[user] = language



import os
import re

from setuptools import setup


def load_requirements(*requirements_paths):
    """
    Load all requirements from the specified requirements files.

    Returns:
        list: Requirements file relative path strings
    """
    requirements = set()
    for path in requirements_paths:
        with open(path) as reqs:
            requirements.update(
                line.split('#')[0].strip() for line in reqs
                if is_requirement(line.strip())
            )
    return list(requirements)


def is_requirement(line):
    """
    Return True if the requirement line is a package requirement.

    Returns:
        bool: True if the line is not blank, a comment, a URL, or an included file
    """
    return line and not line.startswith(('-r', '#', '-e', 'git+', '-c'))


def get_version(*file_paths):
    """
    Extract the version string from the file at the given relative path fragments.
    """
    filename = os.path.join(os.path.dirname(__file__), *file_paths)
    version_file = open(filename).read()
    version_match = re.search(r"^__version__ = ['\"]([^'\"]*)['\"]",
                              version_file, re.M)
    if version_match:
        return version_match.group(1)
    raise RuntimeError('Unable to find version string.')


VERSION = get_version('lang_pref_middleware', '__init__.py')

setup(
    name='django-lang-pref-middleware',
    version=VERSION,
    packages=['lang_pref_middleware'],
    author='edX',
    author_email='oscm@edx.org',
    url='https://github.com/edx/django-lang-pref-middleware',
    description="Django middleware for setting the user's language preference at request time.",
    long_description=open('README.md').read(),
    install_requires=load_requirements('requirements/base.in'),
    classifiers=[
        "Intended Audience :: Developers",
        "License :: OSI Approved :: Apache Software License",
        "Operating System :: OS Independent",
        "Framework :: Django",
        "Framework :: Django :: 2.2",
        "Framework :: Django :: 3.0",
        "Framework :: Django :: 3.1",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.8",
    ]
)



from django.utils.deprecation import MiddlewareMixin


class LanguagePreferenceMiddleware(MiddlewareMixin):
    """
    Middleware for user language preference.

    Sets the user's language preference on the session, if it is not already set.
    """

    def process_request(self, request):
        """Set the language preference, if any, on the session."""
        user = request.user
        if user.is_authenticated and 'django_language' not in request.session:
            user_pref = self.get_user_language_preference(user)
            if user_pref:
                request.session['django_language'] = user_pref

    def get_user_language_preference(self, user):
        """
        Retrieve the given user's language preference.

        Returns None if no preference set.
        """
        raise NotImplementedError   # pragma: no cover
